@@ -33,6 +33,8 @@ import click
 from termcolor import colored
 import daiquiri
 from thoth.python import Project
+from thoth.common import ThothAdviserIntegrationEnum
+from thamos.exceptions import NoProjectDirError
 from thamos.config import config as configuration
 from thamos.lib import advise as thoth_advise
 from thamos.lib import provenance_check as thoth_provenance_check
@@ -120,6 +122,7 @@ def _write_configuration(
     limit_latest_versions: int = None,
     dev: bool = False,
 ) -> None:
+    """Create thoth configuration file."""
     if not advised_configuration:
         _LOGGER.debug("No advises on configuration, nothing to adjust")
         return
@@ -260,13 +263,13 @@ def cli(ctx=None, verbose: bool = False, workdir: str = None, thoth_host: str =
 )
 def _print_version(ctx, json_output: bool = False):
     """Print Thamos and Thoth version and exit."""
-    exit_code = 0
+    api_url = None
     thoth_version = None
     try:
         thoth_version = configuration.get_thoth_version()
-    except Exception:
-        _LOGGER.exception("Failed to obtain version information of Thoth")
-        exit_code = 1
+        api_url = configuration.api_url
+    except NoProjectDirError as exc:
+        _LOGGER.warning("Cannot obtain Thoth backend information: %s", str(exc))
 
     if json_output:
         click.echo(
@@ -281,9 +284,11 @@ def _print_version(ctx, json_output: bool = False):
         )
     else:
         click.echo(f"Thamos Client version: {thamos_version!s}")
-        click.echo(f"Thoth API {configuration.api_url}: {thoth_version}")
+        click.echo(
+            f"Thoth API {api_url if api_url is not None else 'N/A'}: {thoth_version if thoth_version else 'N/A'}"
+        )
 
-    ctx.exit(exit_code)
+    ctx.exit(0)
 
 
 @cli.command("advise")
@@ -377,6 +382,7 @@ def advise(
             debug=debug,
             nowait=no_wait,
             force=force,
+            source_type=ThothAdviserIntegrationEnum.CLI,
             limit_latest_versions=limit_latest_versions,
             no_static_analysis=no_static_analysis,
             dev=dev,
