@@ -43,8 +43,16 @@ class UnknownAnalysisType(ThamosException):
 
 
 class NoRuntimeEnvironmentError(ThamosException):
-    """An exception raised if no runtime environment could be found."""
+    """An exception raised if no runtime environment could be found in configuration file."""
+
+
+class NoRequirementsFormatError(ThamosException):
+    """An exception raised if no requirements format could be found in configuration file."""
 
 
 class ConfigurationError(ThamosException):
     """An exception raised if there are issues with configuration file."""
+
+
+class TimeoutError(ThamosException):
+    """An exception raised if the API takes longer than timeout limit."""
