@@ -35,6 +35,8 @@ from yaspin import yaspin
 from yaspin.spinners import Spinners
 from invectio import gather_library_usage
 from thoth.analyzer import run_command
+from thoth.python import Project
+from thoth.common import ThothAdviserIntegrationEnum
 
 from . import __version__ as thamos_version
 from .swagger_client.rest import ApiException
@@ -49,12 +51,16 @@ from .swagger_client import ImageAnalysisApi
 from .swagger_client import ProvenanceApi
 from .config import config as thoth_config
 from .exceptions import UnknownAnalysisType
+from .exceptions import TimeoutError
 
 urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
 
 LAST_ANALYSIS_ID_FILE = ".thoth_last_analysis_id"
 
 _LOGGER = logging.getLogger(__name__)
+_RETRY_ON_ERROR_COUNT = int(os.getenv("THAMOS_RETRY_ON_ERROR_COUNT", 3))
+_RETRY_ON_ERROR_SLEEP = float(os.getenv("THAMOS_RETRY_ON_ERROR_SLEEP", 3.0))
+_THAMOS_TIMEOUT = int(os.getenv("THAMOS_TIMEOUT", 900))
 
 
 def with_api_client(func: typing.Callable):
@@ -105,14 +111,30 @@ def _wait_for_analysis(status_func: callable, analysis_id: str) -> None:
         spinner = _no_spinner
 
     sleep_time = 0.5
+    retries = 0
     with spinner():
         sleep(2)  # TODO: remove once we fully run on Argo workflows
+        start_time = monotonic()
         while True:
-            response = status_func(analysis_id)
+            if _THAMOS_TIMEOUT and monotonic() - start_time > _THAMOS_TIMEOUT:
+                raise TimeoutError(f"Thoth backend did not respond in time, timeout set to {_THAMOS_TIMEOUT}")
+            try:
+                response = status_func(analysis_id)
+            except Exception as exc:
+                if retries >= _RETRY_ON_ERROR_COUNT:
+                    raise
+
+                retries += 1
+                _LOGGER.error("Failed to obtain status from Thoth: %s", str(exc))
+                _LOGGER.warning("Retrying in a few moments... (attempt %d/%d)", retries, _RETRY_ON_ERROR_COUNT)
+                sleep(_RETRY_ON_ERROR_SLEEP)
+                continue
+
+            retries = 0  # Reset counter as we obtained a valid response.
             if response.status.finished_at is not None:
                 break
             _LOGGER.debug(
-                "Waiting for %r to finish for %s seconds (state: %s)",
+                "Waiting for %r to finish for %g seconds (state: %s)",
                 analysis_id,
                 sleep_time,
                 response.status.state,
@@ -153,16 +175,30 @@ def _retrieve_analysis_result(
     retrieve_func: callable, analysis_id: str
 ) -> typing.Optional[dict]:
     """Retrieve analysis result, raise error if analysis failed."""
-    try:
-        return retrieve_func(analysis_id)
-    except ApiException as exc:
-        _LOGGER.debug(
-            "Retrieved error response %s from server: %s", exc.status, exc.reason
-        )
-        response = json.loads(exc.body)
-        _LOGGER.debug("Error from server: %s", response)
-        _LOGGER.error("%s (analysis: %s)", response["error"], analysis_id)
-        return None
+    retries = 0
+    while True:
+        try:
+            return retrieve_func(analysis_id)
+        except ApiException as exc:
+            _LOGGER.debug(
+                "Retrieved error response %s from server: %s", exc.status, exc.reason
+            )
+            response = json.loads(exc.body)
+
+            if "error" in response:
+                # Error produced based on API endpoints semantics...
+                _LOGGER.debug("Error from Thoth: %s", response)
+                _LOGGER.error("%s (analysis: %s)", response["error"], analysis_id)
+            else:
+                # Other errors (e.g. internal server error).
+                _LOGGER.error("Error from Thoth: %s", response)
+
+            if retries >= _RETRY_ON_ERROR_COUNT:
+                return None
+
+            retries += 1
+            _LOGGER.warning("Retrying in a few moments... (attempt %d/%d)", retries, _RETRY_ON_ERROR_COUNT)
+            sleep(_RETRY_ON_ERROR_SLEEP)
 
 
 def _get_static_analysis() -> typing.Optional[dict]:
@@ -233,6 +269,7 @@ def advise(
     github_check_run_id: typing.Optional[int] = None,
     github_installation_id: typing.Optional[int] = None,
     github_base_repo_url: typing.Optional[str] = None,
+    source_type: typing.Optional[ThothAdviserIntegrationEnum] = None,
 ) -> typing.Optional[tuple]:
     """Submit a stack for adviser checks and wait for results."""
     if not pipfile:
@@ -291,11 +328,14 @@ def advise(
     if recommendation_type:
         recommendation_type = recommendation_type.lower()
 
+    if source_type is None and _is_s2i():
+        source_type = ThothAdviserIntegrationEnum.S2I
+
     parameters = {
         "recommendation_type": recommendation_type,
         "debug": debug,
         "force": force,
-        "is_s2i": _is_s2i(),
+        "source_type": source_type.name,
         "origin": _get_origin(),
         "dev": dev,
     }
@@ -354,54 +394,58 @@ def advise(
 
 
 def advise_here(
-    recommendation_type: str = None,
+    recommendation_type: typing.Optional[str] = None,
     *,
     runtime_environment: dict = None,
-    runtime_environment_name: str = None,
-    limit_latest_versions: int = None,
+    runtime_environment_name: typing.Optional[str] = None,
+    limit_latest_versions: typing.Optional[int] = None,
     dev: bool = False,
     no_static_analysis: bool = False,
     nowait: bool = False,
     force: bool = False,
-    limit: int = None,
+    limit: typing.Optional[int] = None,
     count: int = 1,
     debug: bool = False,
-    origin: str = None,
+    origin: typing.Optional[str] = None,
     github_event_type: typing.Optional[str] = None,
     github_check_run_id: typing.Optional[int] = None,
     github_installation_id: typing.Optional[int] = None,
-    github_base_repo_url: typing.Optional[str] = None
+    github_base_repo_url: typing.Optional[str] = None,
+    source_type: typing.Optional[ThothAdviserIntegrationEnum] = None,
 ) -> typing.Optional[tuple]:
     """Run advise in current directory, requires no arguments."""
-    if not os.path.isfile("Pipfile"):
-        raise FileNotFoundError("No Pipfile found in current directory")
+    requirements_format = thoth_config.requirements_format
+    if requirements_format == "pipenv":
+        project = Project.from_files(without_pipfile_lock=not os.path.exists("Pipfile.lock"))
+    elif requirements_format in ("pip", "pip-tools", "pip-compile"):
+        project = Project.from_pip_compile_files(allow_without_lock=True)
+    else:
+        raise ValueError(f"Unknown configuration option for requirements format: {requirements_format!r}")
 
-    with open("Pipfile", "r") as pipfile:
-        pipfile_lock_str = ""
-        if os.path.isfile("Pipfile.lock"):
-            with open("Pipfile.lock", "r") as pipfile_lock:
-                pipfile_lock_str = pipfile_lock.read()
+    pipfile = project.pipfile.to_string()
+    pipfile_lock_str = project.pipfile_lock.to_string() if project.pipfile_lock else ""
 
-        return advise(
-            pipfile=pipfile.read(),
-            pipfile_lock=pipfile_lock_str,
-            recommendation_type=recommendation_type,
-            runtime_environment=runtime_environment,
-            runtime_environment_name=runtime_environment_name,
-            limit_latest_versions=limit_latest_versions,
-            dev=dev,
-            no_static_analysis=no_static_analysis,
-            nowait=nowait,
-            force=force,
-            limit=limit,
-            count=count,
-            debug=debug,
-            origin=origin,
-            github_event_type=github_event_type,
-            github_check_run_id=github_check_run_id,
-            github_installation_id=github_installation_id,
-            github_base_repo_url=github_base_repo_url,
-        )
+    return advise(
+        pipfile=pipfile,
+        pipfile_lock=pipfile_lock_str,
+        recommendation_type=recommendation_type,
+        runtime_environment=runtime_environment,
+        runtime_environment_name=runtime_environment_name,
+        limit_latest_versions=limit_latest_versions,
+        dev=dev,
+        no_static_analysis=no_static_analysis,
+        nowait=nowait,
+        force=force,
+        limit=limit,
+        count=count,
+        debug=debug,
+        origin=origin,
+        source_type=source_type.name,
+        github_event_type=github_event_type,
+        github_check_run_id=github_check_run_id,
+        github_installation_id=github_installation_id,
+        github_base_repo_url=github_base_repo_url,
+    )
 
 
 @with_api_client
@@ -630,7 +674,7 @@ def get_log(api_client: ApiClient, analysis_id: str = None):
         except Exception:
             # If the content parsed does not carry logger information or has not relevant
             # entries, log the original message.
-            result += line
+            result += line + "\n"
             continue
 
     return result
@@ -678,9 +722,7 @@ def get_analysis_results(api_client: ApiClient, analysis_id: str):
     elif analysis_id.startswith("adviser-"):
         api_instance = AdviseApi(api_client)
         method = api_instance.get_advise_python
-        response = _retrieve_analysis_result(
-            api_instance.get_advise_python, analysis_id
-        )
+        response = _retrieve_analysis_result(method, analysis_id)
         return response.result, response.result["error"]
     else:
         raise UnknownAnalysisType(
